/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.container;

import java.util.List;

import de.bsvrz.ibv.uda.interpreter.AbstractUdaHandler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.Fehler;
import de.bsvrz.sys.funclib.bitctrl.interpreter.HandlerValidation;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Handler für Operationen mit Uda-Containerobjekten.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ContainerHandler extends AbstractUdaHandler {

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * {@inheritDoc}.
	 *
	 * @return die Liste der unterstützten Operatoren
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#getHandledOperators()
	 */
	@Override
	public Operator[] getHandledOperators() {
		return ContainerOperators.OPERATOREN;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#validiereHandler(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public HandlerValidation validiereHandler(final Operator operator, final List<? extends Object> operanden) {

		boolean anzahlOk = false;
		boolean typOk = false;
		final int operandenAnzahl = operanden.size();

		if (ContainerOperators.FELD_KONSTRUKTOR.equals(operator)) {
			if (operandenAnzahl == 0) {
				anzahlOk = true;
				typOk = true;
			} else if (operandenAnzahl == 1) {
				anzahlOk = true;
				final Object parameter = operanden.get(0);
				if (parameter instanceof Number) {
					typOk = true;
				} else if (parameter instanceof UdaContainer) {
					typOk = true;
				}
			}
		} else if (ContainerOperators.LISTEN_KONSTRUKTOR.equals(operator)) {
			if ((operandenAnzahl == 0) || ((operandenAnzahl == 1) && (operanden.get(0) instanceof UdaContainer))) {
				anzahlOk = true;
				typOk = true;
			}
		} else if (ContainerOperators.SCHLUSSELMENGEN_KONSTRUKTOR.equals(operator)) {
			if (operandenAnzahl == 0) {
				anzahlOk = true;
				typOk = true;
			}
		} else if (ContainerOperators.ANZAHL.equals(operator)) {
			if (operandenAnzahl == 0) {
				anzahlOk = true;
				typOk = true;
			}
		} else if (ContainerOperators.EINFUEGEN.equals(operator)) {
			if (operanden.size() > 1) {
				anzahlOk = true;
				final Object objekt = operanden.get(0);
				if ((objekt instanceof Container) || (objekt instanceof UdaIterator)) {
					typOk = true;
				}
			}
		} else if (ContainerOperators.ANHAENGEN.equals(operator)) {
			if (operanden.size() > 1) {
				anzahlOk = true;
				final Object objekt = operanden.get(0);
				if ((objekt instanceof Container) || (objekt instanceof UdaIterator)) {
					typOk = true;
				}
			}
		} else if (ContainerOperators.ELEMENT.equals(operator)) {
			if (operanden.size() > 0) {
				anzahlOk = true;
				final Object objekt = operanden.get(0);
				if ((objekt instanceof Container) || (objekt instanceof UdaIterator)) {
					typOk = true;
				}
			}
		} else if (ContainerOperators.LOESCHEN.equals(operator)) {
			if (operanden.size() > 0) {
				anzahlOk = true;
				final Object objekt = operanden.get(0);
				if ((objekt instanceof Container) || (objekt instanceof UdaIterator)) {
					typOk = true;
				}
			}
		} else if (ContainerOperators.ANFANG.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				final Object objekt = operanden.get(0);
				if (objekt instanceof Container) {
					typOk = true;
				}
			}
		} else if (ContainerOperators.ERSTES.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				final Object objekt = operanden.get(0);
				if (objekt instanceof Container) {
					typOk = true;
				}
			}
		} else if (ContainerOperators.ITERATOR.equals(operator)) {
			if (operanden.size() > 1) {
				anzahlOk = true;
				final Object objekt = operanden.get(0);
				if (objekt instanceof Container) {
					typOk = true;
				}
			}
		} else if (ContainerOperators.LETZTES.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				final Object objekt = operanden.get(0);
				if (objekt instanceof Container) {
					typOk = true;
				}
			}
		} else if (ContainerOperators.SCHLUSS.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				final Object objekt = operanden.get(0);
				if (objekt instanceof Container) {
					typOk = true;
				}
			}
		} else if (ContainerOperators.CONTAINER.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				final Object objekt = operanden.get(0);
				if (objekt instanceof UdaIterator) {
					typOk = true;
				}
			}
		} else if (ContainerOperators.VORGAENGER.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				final Object objekt = operanden.get(0);
				if (objekt instanceof UdaIterator) {
					typOk = true;
				}
			}
		} else if (ContainerOperators.NACHFOLGER.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				final Object objekt = operanden.get(0);
				if (objekt instanceof UdaIterator) {
					typOk = true;
				}
			}
		} else if (ContainerOperators.WEITER.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				final Object objekt = operanden.get(0);
				if (objekt instanceof UdaIterator) {
					typOk = true;
				}
			}
		} else if (ContainerOperators.ZURUECK.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				final Object objekt = operanden.get(0);
				if (objekt instanceof UdaIterator) {
					typOk = true;
				}
			}
		} else if (ContainerOperators.INNERHALB.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				final Object objekt = operanden.get(0);
				if (objekt instanceof UdaIterator) {
					typOk = true;
				}
			}
		} else if (ContainerOperators.SCHLUESSEL.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				final Object objekt = operanden.get(0);
				if (objekt instanceof UdaIterator) {
					typOk = true;
				}
			}
		}

		return new HandlerValidation(anzahlOk, typOk);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#perform(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public Object perform(final Operator operator, final List<Object> operanden) {

		erzeugeHandlerFehler(operator, operanden);
		Object result = null;
		try {
			if (ContainerOperators.FELD_KONSTRUKTOR.equals(operator)) {
				Object initialisierung = null;
				if (operanden.size() > 0) {
					initialisierung = operanden.get(0);
					if (initialisierung instanceof Number) {
						result = new Feld((Number) initialisierung);
					} else if (initialisierung instanceof UdaContainer) {
						result = new Feld((UdaContainer) initialisierung);
					}
				} else {
					result = new Feld();
				}
			} else if (ContainerOperators.LISTEN_KONSTRUKTOR.equals(operator)) {
				UdaContainer initialisierung = null;
				if (operanden.size() > 0) {
					initialisierung = (UdaContainer) operanden.get(0);
				}
				result = new Liste(initialisierung);
			} else if (ContainerOperators.SCHLUSSELMENGEN_KONSTRUKTOR.equals(operator)) {
				result = new SchluesselMenge();
			} else if (ContainerOperators.EINFUEGEN.equals(operator)) {
				result = elementEinfuegen(operanden);
			} else if (ContainerOperators.ANHAENGEN.equals(operator)) {
				result = elementAnhaengen(operanden);
			} else if (ContainerOperators.ELEMENT.equals(operator)) {
				final Object obj = operanden.get(0);
				if (obj instanceof SchluesselMenge) {
					if (operanden.size() > 1) {
						final SchluesselMenge menge = (SchluesselMenge) obj;
						result = menge.getContainerElement(operanden.get(1).toString());
					}
				} else if (obj instanceof Container) {
					if (operanden.size() > 1) {
						final Container container = (Container) obj;
						final int index = ((Number) operanden.get(1)).intValue();
						result = container.getContainerElement(index);
					}
				} else if (obj instanceof UdaIterator) {
					final UdaIterator iterator = (UdaIterator) obj;
					result = iterator.getElement();
					if (operanden.size() > 1) {
						iterator.setElement(operanden.get(1));
					}
				}
			} else if (ContainerOperators.LOESCHEN.equals(operator)) {
				result = loescheContainerElement(operanden);
			} else if (ContainerOperators.ANFANG.equals(operator)) {
				UdaContainer container = null;
				if (operanden.size() > 0) {
					container = (UdaContainer) operanden.get(0);
					result = container.getUdaIterator(UdaIterator.ANFANG);
				}
			} else if (ContainerOperators.ERSTES.equals(operator)) {
				UdaContainer container = null;
				if (operanden.size() > 0) {
					container = (UdaContainer) operanden.get(0);
					result = container.getUdaIterator(1);
				}
			} else if (ContainerOperators.ITERATOR.equals(operator)) {
				UdaContainer container = null;
				if (operanden.size() > 1) {
					container = (UdaContainer) operanden.get(0);
					result = container.getUdaIterator(operanden.get(1));
				}
			} else if (ContainerOperators.LETZTES.equals(operator)) {
				UdaContainer container = null;
				if (operanden.size() > 0) {
					container = (UdaContainer) operanden.get(0);
					result = container.getUdaIterator(container.getGroesse());
				}
			} else if (ContainerOperators.SCHLUSS.equals(operator)) {
				UdaContainer container = null;
				if (operanden.size() > 0) {
					container = (UdaContainer) operanden.get(0);
					result = container.getUdaIterator(container.getGroesse() + 1);
				}
			} else if (ContainerOperators.CONTAINER.equals(operator)) {
				UdaIterator iterator = null;
				if (operanden.size() > 0) {
					iterator = (UdaIterator) operanden.get(0);
					result = iterator.getContainer();
				}
			} else if (ContainerOperators.VORGAENGER.equals(operator)) {
				UdaIterator iterator = null;
				if (operanden.size() > 0) {
					iterator = (UdaIterator) operanden.get(0);
					result = iterator.getVorgaengerIterator();
				}
			} else if (ContainerOperators.NACHFOLGER.equals(operator)) {
				UdaIterator iterator = null;
				if (operanden.size() > 0) {
					iterator = (UdaIterator) operanden.get(0);
					result = iterator.getNachfolgerIterator();
				}
			} else if (ContainerOperators.ZURUECK.equals(operator)) {
				UdaIterator iterator = null;
				if (operanden.size() > 0) {
					iterator = (UdaIterator) operanden.get(0);
					result = iterator.zurueck();
				}
			} else if (ContainerOperators.WEITER.equals(operator)) {
				UdaIterator iterator = null;
				if (operanden.size() > 0) {
					iterator = (UdaIterator) operanden.get(0);
					result = iterator.weiter();
				}
			} else if (ContainerOperators.INNERHALB.equals(operator)) {
				UdaIterator iterator = null;
				if (operanden.size() > 0) {
					iterator = (UdaIterator) operanden.get(0);
					result = iterator.innerhalb();
				}
			} else if (ContainerOperators.SCHLUESSEL.equals(operator)) {
				UdaIterator iterator = null;
				if (operanden.size() > 0) {
					iterator = (UdaIterator) operanden.get(0);
					result = iterator.schluessel();
				}
			}
		} catch (final ClassCastException e) {
			final String message = "Der Operator \"" + operator.getSymbol()
			+ "\" ist auf den Typ \"" + operanden.get(0).getClass()
			+ "\" nicht anwendbar";
			LOGGER.error(message, e);
			throw new Fehler(message);
		}
		return result;
	}

	/**
	 * löscht ein Element aus einem Container.
	 *
	 * @param operanden
	 *            die Operanden, die an die Opration übergeben wurden.
	 * @return das Ergebnis der Operation (das gelöschte Objekt)
	 */
	private Object loescheContainerElement(final List<Object> operanden) {
		Object result = null;
		if (operanden.size() > 0) {
			final Object containerObjekt = operanden.get(0);
			if (containerObjekt instanceof SchluesselMenge) {
				if (operanden.size() > 1) {
					result = ((SchluesselMenge) containerObjekt).loeschen(operanden.get(1));
				}
			} else {
				if (operanden.size() > 1) {
					UdaContainer container = null;
					container = (UdaContainer) operanden.get(0);
					final int index = ((Number) operanden.get(1)).intValue();
					result = container.loeschen(index);
				} else {
					final UdaIterator iterator = (UdaIterator) operanden.get(0);
					if (iterator.innerhalb().getBoolWert()) {
						// result = iterator.getElement();
						result = iterator.getContainer().loeschen(iterator.getPosition());
					}
				}
			}
		}
		return result;
	}

	/**
	 * führt eine Einfügeoperation mit den übergebenen Operanden aus.
	 *
	 * @param operanden
	 *            die Operanden
	 * @return das eingefügte Objekt
	 */
	private Object elementEinfuegen(final List<Object> operanden) {
		Object result = null;

		if (operanden.size() > 0) {
			final Object destination = operanden.get(0);
			if (destination instanceof SchluesselMenge) {
				if (operanden.size() > 2) {
					result = ((SchluesselMenge) destination).einfuegen(operanden.get(1), operanden.get(2));
				}
			} else if (operanden.size() > 1) {
				int index = 1;
				UdaContainer container = null;
				Object daten = null;

				if (destination instanceof UdaIterator) {
					final UdaIterator iterator = (UdaIterator) destination;
					container = iterator.getContainer();
					daten = operanden.get(1);
					switch (iterator.getPosition()) {
					case UdaIterator.ANFANG:
						index = 1;
						break;
					case UdaIterator.ENDE:
						index = container.getGroesse();
						break;
					default:
						index = iterator.getPosition();
						break;
					}
				} else {
					container = (UdaContainer) operanden.get(0);
					if (operanden.size() > 2) {
						index = ((Number) operanden.get(1)).intValue();
						daten = operanden.get(2);
					} else {
						daten = operanden.get(1);
					}
				}

				result = container.einfuegen(index, daten);
				if (destination instanceof UdaIterator) {
					result = container.getUdaIterator(index);
				}
			}
		}

		return result;
	}

	/**
	 * führt eine Anhängeoperation in einem Container mit den gegebenen
	 * Operanden aus.
	 *
	 * @param operanden
	 *            die Operanden
	 * @return das angehängte Objekt
	 */
	private Object elementAnhaengen(final List<Object> operanden) {
		Object result = null;
		if (operanden.size() > 1) {
			int index = 1;
			final Object obj = operanden.get(0);
			UdaContainer container = null;
			Object daten = null;

			if (obj instanceof UdaIterator) {
				final UdaIterator iterator = (UdaIterator) obj;
				container = iterator.getContainer();
				daten = operanden.get(1);
				switch (iterator.getPosition()) {
				case UdaIterator.ANFANG:
					index = 0;
					break;
				case UdaIterator.ENDE:
					index = container.getGroesse();
					break;
				default:
					index = iterator.getPosition();
					break;
				}
			} else {
				container = (UdaContainer) operanden.get(0);
				index = -1;
				if (operanden.size() > 2) {
					index = ((Number) operanden.get(1)).intValue();
					daten = operanden.get(2);
				} else {
					daten = operanden.get(1);
				}
			}
			container.anhaengen(index, daten);
			result = daten;
			if (obj instanceof UdaIterator) {
				result = container.getUdaIterator(index + 1);
			}
		}
		return result;
	}

}
