/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.dav;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.Aspect;

/**
 * die Repräsentation eines Datenverteiler-<i>Aspect</i>-Objekts in einem
 * Uda-Skript.<br>
 * Das Objekt hat, bis auf die vom Typ <i>KonfigurationsObjekt</i> vererbten:
 * <ul>
 * <li>pid</li>
 * <li>id</li>
 * <li>name</li>
 * <li>info</li>
 * <li>typ</li>
 * <li>gültig</li>
 * <li>attributgruppen</li>
 * <li>mengen</li>
 * </ul>
 * keine zusätzlichen Attribute.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class AspektObjekt extends KonfigurationsObjekt {

	/**
	 * Konstruktor.
	 *
	 * @param verbindung
	 *            die verwendete Datenverteilerverbindung
	 * @param object
	 *            das <i>Aspect</i>-Objekt des Datenverteilers
	 */
	public AspektObjekt(final ClientDavInterface verbindung, final Aspect object) {
		super(verbindung, object);
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion liefert immer ein <i>Aspect</i>-Objekt im Gegensatz zur
	 * allgemeineren Definition in <i>DavDaten</i>.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.dav.DavDaten#getObjekt()
	 */
	@Override
	public Aspect getObjekt() {
		return (Aspect) super.getObjekt();
	}
}
