/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.trend;

import java.util.List;

import de.bsvrz.ibv.uda.interpreter.AbstractUdaHandler;
import de.bsvrz.ibv.uda.interpreter.daten.zeit.ZeitDauer;
import de.bsvrz.ibv.uda.interpreter.daten.zeit.Zeitstempel;
import de.bsvrz.sys.funclib.bitctrl.interpreter.HandlerValidation;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;

/**
 * Handler f&uuml;r die Verarbietung von Trendobjekten.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class TrendHandler extends AbstractUdaHandler {

	/**
	 * {@inheritDoc}.
	 *
	 * @return die Liste der unterstützten Operatoren
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#getHandledOperators()
	 */
	@Override
	public Operator[] getHandledOperators() {
		return TrendOperatoren.TREND_OPERATOREN;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#validiereHandler(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public HandlerValidation validiereHandler(final Operator operator, final List<? extends Object> operanden) {
		boolean anzahlOK = false;
		boolean typOK = false;

		final int anzahlOperanden = operanden.size();

		if (TrendOperatoren.KONSTRUKTOR.equals(operator)) {
			anzahlOK = true;
			typOK = true;
			if (anzahlOperanden > 1) {
				if ((!(operanden.get(1) instanceof Number)) || (operanden.size() > 2)) {
					typOK = false;
				}
			}
		} else if (TrendOperatoren.BEZUG.equals(operator)) {
			if (anzahlOperanden == 1) {
				anzahlOK = true;
				if (operanden.get(0) instanceof TrendObjekt) {
					typOK = true;
				}
			}
		} else if (TrendOperatoren.ANZAHL.equals(operator)) {
			if (anzahlOperanden == 1) {
				anzahlOK = true;
				if (operanden.get(0) instanceof TrendObjekt) {
					typOK = true;
				}
			}
		} else if (TrendOperatoren.WERT.equals(operator)) {
			if (anzahlOperanden > 0) {
				anzahlOK = true;
				if (operanden.get(0) instanceof TrendObjekt) {
					switch (anzahlOperanden) {
					case 1:
						typOK = true;
						break;
					case 2:
						if (operanden.get(1) instanceof Number) {
							typOK = true;
						}
						break;
					case 3:
						if ((operanden.get(1) instanceof Number) && (operanden.get(2) instanceof Zeitstempel)) {
							typOK = true;
						}
						break;
					default:
						anzahlOK = false;
						break;
					}
				}
			}
		} else if (TrendOperatoren.ZEIT.equals(operator)) {
			if (anzahlOperanden == 1) {
				anzahlOK = true;
				if (operanden.get(0) instanceof TrendObjekt) {
					typOK = true;
				}
			}
		} else if (TrendOperatoren.MAXFALLEND.equals(operator)) {
			if (anzahlOperanden == 1) {
				anzahlOK = true;
				if (operanden.get(0) instanceof TrendObjekt) {
					typOK = true;
				}
			}
		} else if (TrendOperatoren.MAXSTEIGEND.equals(operator)) {
			if (anzahlOperanden == 1) {
				anzahlOK = true;
				if (operanden.get(0) instanceof TrendObjekt) {
					typOK = true;
				}
			}
		} else if (TrendOperatoren.MAXKONSTANT.equals(operator)) {
			if (anzahlOperanden == 1) {
				anzahlOK = true;
				if (operanden.get(0) instanceof TrendObjekt) {
					typOK = true;
				}
			}
		} else if (TrendOperatoren.STEIGEND.equals(operator) || TrendOperatoren.FALLEND.equals(operator)
				|| TrendOperatoren.KONSTANT.equals(operator)) {
			if (anzahlOperanden > 0) {
				anzahlOK = true;
				if (operanden.get(0) instanceof TrendObjekt) {
					if (anzahlOperanden > 1) {
						if ((operanden.get(1) instanceof Number) || (operanden.get(1) instanceof ZeitDauer)) {
							if (operanden.size() == 3) {
								if (operanden.get(2) instanceof Zeitstempel) {
									typOK = true;
								}
							} else if (operanden.size() == 2) {
								typOK = true;
							}
						}
					} else {
						typOK = true;
					}
				}
			}
		}

		return new HandlerValidation(anzahlOK, typOK);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#perform(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public Object perform(final Operator operator, final List<Object> operanden) {
		erzeugeHandlerFehler(operator, operanden);

		Object result;
		result = null;
		if (operator.equals(TrendOperatoren.KONSTRUKTOR)) {
			Object bezug = NichtWert.NICHTWERT;
			int kapazitaet = 0;
			if (operanden.size() > 0) {
				bezug = operanden.get(0);
			}
			if (operanden.size() > 1) {
				kapazitaet = ((Number) operanden.get(1)).intValue();
			}
			result = new TrendObjekt(bezug, kapazitaet);
		} else {
			final TrendObjekt trend = (TrendObjekt) operanden.get(0);
			if (operator.equals(TrendOperatoren.BEZUG)) {
				result = trend.getBezug();
			} else if (operator.equals(TrendOperatoren.ANZAHL)) {
				result = trend.getAnzahl();
			} else if (operator.equals(TrendOperatoren.WERT)) {
				if (operanden.size() > 1) {
					final Number wert = (Number) operanden.get(1);
					Zeitstempel zeitstempel = null;
					if (operanden.size() > 2) {
						zeitstempel = (Zeitstempel) operanden.get(2);
					}
					trend.addWert(wert, zeitstempel);
				}
				result = trend.getWert();
			} else if (operator.equals(TrendOperatoren.ZEIT)) {
				result = trend.getZeit();
			} else if (operator.equals(TrendOperatoren.FALLEND)) {
				Object parameter = null;
				Zeitstempel zeitstempel = null;
				if (operanden.size() > 1) {
					parameter = operanden.get(1);
				}
				if (operanden.size() == 3) {
					zeitstempel = (Zeitstempel) operanden.get(2);
				}
				result = trend.isFallend(parameter, zeitstempel);
			} else if (operator.equals(TrendOperatoren.STEIGEND)) {
				Object parameter = null;
				Zeitstempel zeitstempel = null;
				if (operanden.size() > 1) {
					parameter = operanden.get(1);
				}
				if (operanden.size() == 3) {
					zeitstempel = (Zeitstempel) operanden.get(2);
				}
				result = trend.isSteigend(parameter, zeitstempel);
			} else if (operator.equals(TrendOperatoren.KONSTANT)) {
				Object parameter = null;
				Zeitstempel zeitstempel = null;
				if (operanden.size() > 1) {
					parameter = operanden.get(1);
				}
				if (operanden.size() == 3) {
					zeitstempel = (Zeitstempel) operanden.get(2);
				}
				result = trend.isKonstant(parameter, zeitstempel);
			} else if (operator.equals(TrendOperatoren.MAXFALLEND)) {
				result = trend.getMaxFallend();
			} else if (operator.equals(TrendOperatoren.MAXKONSTANT)) {
				result = trend.getMaxKonstant();
			} else if (operator.equals(TrendOperatoren.MAXSTEIGEND)) {
				result = trend.getMaxSteigend();
			}
		}
		return result;
	}
}
